import platform
import pytest
import math
import numpy as np
import dartpy as dart


def test_inertia_init():
    ''' 
    Test basic functionality for the `dartpy.dynamics.Inertia` class.
    '''
    # test default values
    i1 = dart.dynamics.Inertia()
    assert i1 is not None

    # initialize with parameters
    i2 = dart.dynamics.Inertia(0.1, [0, 0, 0], 1.3*np.eye(3))
    assert i1 is not None

    newMass = 1.5
    i2.setMass(newMass)
    assert i2.getMass() == newMass

    newCOM = np.array((0.1, 0, 0))
    i2.setLocalCOM(newCOM)
    assert np.allclose(i2.getLocalCOM(), newCOM)

    newMoment = 0.4*newMass*0.1**2*np.eye(3)
    i2.setMoment(newMoment)
    assert np.allclose(i2.getMoment(), newMoment)

    i2.setSpatialTensor(0.3*i2.getSpatialTensor())

    assert i2.verify()

    for i in range(10):  # based on the C++ tests
        mass = np.random.uniform(0.1, 10.0)
        com = np.random.uniform(-5, 5, 3)
        I = np.random.rand(3, 3) - 0.5 + \
            np.diag(np.random.uniform(0.6, 1, 3), 0)
        I = (I + I.T)/2

        inertia = dart.dynamics.Inertia(mass, com, I)
        assert inertia.verify()


def test_inertia_static_methods():
    '''
    Test the class methods `verifyMoment`and `verifySpatialTensor`.
    '''
    assert dart.dynamics.Inertia.verifyMoment(np.eye(3), printWarnings=False)
    for i in range(10):
        I = np.random.rand(3, 3) - 0.5 + \
            np.diag(np.random.uniform(1, 10, 3), 0)
        I = (I + I.T)/2
        assert dart.dynamics.Inertia.verifyMoment(I)

    assert dart.dynamics.Inertia.verifySpatialTensor(
        np.eye(6), printWarnings=False)


def test_failing_moment_and_spatial():
    '''
    Test some failure cases of the verify methods.
    '''

    for i in range(10):
        I = np.random.rand(3, 3) - 0.5 - \
            np.diag(np.random.uniform(1, 10, 3), 0)
        assert not dart.dynamics.Inertia.verifyMoment(I, printWarnings=False)

    # fails e.g. due to off diagonal values in translational part.
    assert not dart.dynamics.Inertia.verifySpatialTensor(
        np.random.rand(6, 6), printWarnings=False)


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
import dartpy as dart


# TODO(JS): Move this to integration category once created


def test_basic():
    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    assert kr5 is not None

    shoulder = kr5.getBodyNode('shoulder')
    assert shoulder is not None

    elbow = kr5.getBodyNode('elbow')
    assert elbow is not None

    chain1 = dart.dynamics.Chain(shoulder, elbow, False, 'midchain')
    assert chain1 is not None
    assert chain1.getNumBodyNodes() is 2

    chain2 = dart.dynamics.Chain(shoulder, elbow, True, 'midchain')
    assert chain2 is not None
    assert chain2.getNumBodyNodes() is 3

    assert len(kr5.getPositions()) is not 0
    assert kr5.getNumJoints() is not 0
    assert kr5.getRootJoint() is not None
    assert len(kr5.getRootJoint().getPositions()) is 0

    rootBody = kr5.getBodyNode(0)
    assert rootBody is not None

    rootJoint = kr5.getJoint(0)
    assert rootJoint is not None


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
import dartpy as dart


def test_empty_world():
    world = dart.simulation.World('my world')
    assert world.getNumSkeletons() is 0
    assert world.getNumSimpleFrames() is 0


def test_collision_detector_change():
    world = dart.simulation.World('world')
    solver = world.getConstraintSolver()
    assert solver is not None

    assert solver.getCollisionDetector().getType() == dart.collision.FCLCollisionDetector().getStaticType()

    solver.setCollisionDetector(dart.collision.DARTCollisionDetector())
    assert solver.getCollisionDetector().getType() == dart.collision.DARTCollisionDetector().getStaticType()

    if hasattr(dart.collision, 'BulletCollisionDetector'):
        solver.setCollisionDetector(dart.collision.BulletCollisionDetector())
        assert solver.getCollisionDetector().getType() == dart.collision.BulletCollisionDetector().getStaticType()

    if hasattr(dart.collision, 'OdeCollisionDetector'):
        solver.setCollisionDetector(dart.collision.OdeCollisionDetector())
        assert solver.getCollisionDetector().getType() == dart.collision.OdeCollisionDetector().getStaticType()


if __name__ == "__main__":
    pytest.main()



import numpy as np
import dartpy as dart


def main():
    world = dart.simulation.World()

    tf = dart.math.Isometry3()

    tf.set_translation([4, -4, 0])
    frame = dart.gui.osg.InteractiveFrame(
        dart.dynamics.Frame.World(), 'interactive frame', tf, 2)
    world.addSimpleFrame(frame)

    tf.set_translation([-4, 4, 0])
    draggable = dart.dynamics.SimpleFrame(frame, 'draggable', tf)
    draggable.setShape(dart.dynamics.BoxShape([1, 1, 1]))
    draggable.getVisualAspect(True).setColor([0.9, 0, 0])
    world.addSimpleFrame(draggable)

    tf.set_translation([8, 0, 0])
    x_marker = dart.dynamics.SimpleFrame(dart.dynamics.Frame.World(), 'X', tf)
    x_shape = dart.dynamics.BoxShape([0.2, 0.2, 0.2])
    x_marker.setShape(x_shape)
    x_marker.getVisualAspect(True).setColor([0.9, 0, 0])
    world.addSimpleFrame(x_marker)

    tf.set_translation([0, 8, 0])
    y_marker = dart.dynamics.SimpleFrame(dart.dynamics.Frame.World(), 'Y', tf)
    y_shape = dart.dynamics.BoxShape([0.2, 0.2, 0.2])
    y_marker.setShape(y_shape)
    y_marker.getVisualAspect(True).setColor([0, 0.9, 0])
    world.addSimpleFrame(y_marker)

    tf.set_translation([0, 0, 8])
    z_marker = dart.dynamics.SimpleFrame(dart.dynamics.Frame.World(), 'Z', tf)
    z_shape = dart.dynamics.BoxShape([0.2, 0.2, 0.2])
    z_marker.setShape(z_shape)
    z_marker.getVisualAspect(True).setColor([0, 0, 0.9])
    world.addSimpleFrame(z_marker)

    node = dart.gui.osg.WorldNode(world)

    viewer = dart.gui.osg.Viewer()
    viewer.addWorldNode(node)
    viewer.enableDragAndDrop(frame)
    viewer.enableDragAndDrop(draggable)

    viewer.addInstructionText("\nCtrl + Left-click: Rotate the box\n")
    print(viewer.getInstructions())

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([20, 17, 17], [0, 0, 0], [0, 0, 1])
    viewer.run()


if __name__ == "__main__":
    main()



import pytest
import dartpy as dart


def test_basics():
    sw = dart.common.Stopwatch()

    # Stopwatch is started by default
    assert sw.isStarted()
    assert dart.common.Stopwatch(True).isStarted()
    assert not dart.common.Stopwatch(False).isStarted()

    # Stop the stopwatch
    sw.stop()
    assert not sw.isStarted()

    # Elapsed time should be the same
    elapsed1 = sw.elapsedS()
    assert elapsed1 == sw.elapsedS()
    assert elapsed1 == sw.elapsedS()

    # Elapsed time monotonically increase while the stopwatch is running
    sw.start()
    assert sw.elapsedS() >= elapsed1
    assert sw.elapsedS() >= elapsed1

    # Starting a stopwatch already started doesn't have any effect
    sw.start()
    assert sw.isStarted()

    # Restting a started stopwatch resets the elapsed time but doesn't stop the
    # stopwatch
    sw.start()
    sw.reset()
    assert sw.isStarted()
    assert sw.elapsedS() >= 0.0
    assert sw.elapsedS() >= 0.0

    # Restting a stopped stopwatch resets the elapsed time but doesn't start the
    # stopwatch
    sw.stop()
    sw.reset()
    assert not sw.isStarted()
    assert sw.elapsedS() == pytest.approx(0.0)

    sw.print()


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
import numpy as np
import dartpy as dart


def test_basic():
    skel = dart.dynamics.Skeleton()

    joint_prop = dart.dynamics.FreeJointProperties()
    joint_prop.mName = 'joint0'
    assert joint_prop.mName == 'joint0'

    [joint1, body1] = skel.createFreeJointAndBodyNodePair(None, joint_prop)
    assert joint1.getType() == 'FreeJoint'
    assert joint1.getName() == 'joint0'

    assert skel.getBodyNodes()[0].getName() == body1.getName()


if __name__ == "__main__":
    pytest.main()



import math
import pytest
import numpy as np
import dartpy as dart


def kinematics_tester(joint):
    num_tests = 2

    joint.setTransformFromChildBodyNode(dart.math.expMap(np.random.rand(6)))
    joint.setTransformFromParentBodyNode(dart.math.expMap(np.random.rand(6)))

    dof = joint.getNumDofs()

    q = np.zeros(dof)
    dq = np.zeros(dof)

    for _ in range(num_tests):
        q_delta = 1e-5

        for i in range(dof):
            q[i] = dart.math.Random.uniform(-math.pi, math.pi)
            dq[i] = dart.math.Random.uniform(-math.pi, math.pi)

        joint.setPositions(q)
        joint.setVelocities(dq)

        if dof == 0:
            return

        T = joint.getRelativeTransform()
        J = joint.getRelativeJacobian(q)
        dJ = joint.getRelativeJacobianTimeDeriv()

        # Verify transform
        assert dart.math.verifyTransform(T)

        # Test analytic Jacobian and numerical Jacobian
        numericJ = np.zeros((6, dof))
        for i in range(dof):
            q_a = q.copy()
            joint.setPositions(q_a)
            T_a = joint.getRelativeTransform()

            q_b = q.copy()
            q_b[i] += q_delta
            joint.setPositions(q_b)
            T_b = joint.getRelativeTransform()

            Tinv_a = T_a.inverse()

            dTdq = (T_b.matrix() - T_a.matrix()) / q_delta

            Ji_4x4matrix = np.matmul(Tinv_a.matrix(), dTdq)
            Ji = np.zeros(6)
            Ji[0] = Ji_4x4matrix[2, 1]
            Ji[1] = Ji_4x4matrix[0, 2]
            Ji[2] = Ji_4x4matrix[1, 0]
            Ji[3] = Ji_4x4matrix[0, 3]
            Ji[4] = Ji_4x4matrix[1, 3]
            Ji[5] = Ji_4x4matrix[2, 3]
            numericJ[:, i] = Ji

        assert np.allclose(J, numericJ, atol=1e-5)


def test_kinematics():
    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createWeldJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createRevoluteJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createPrismaticJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createScrewJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createUniversalJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createTranslationalJoint2DAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createEulerJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createTranslationalJointAndBodyNodePair()
    kinematics_tester(joint)

    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createPlanarJointAndBodyNodePair()
    kinematics_tester(joint)


def test_access_to_parent_child_transforms():
    skel = dart.dynamics.Skeleton()
    joint, _ = skel.createRevoluteJointAndBodyNodePair()

    parentToJointTf = dart.math.Isometry3.Identity()
    parentToJointTf.set_translation(np.random.rand(3, 1))
    childToJointTf = dart.math.Isometry3.Identity()
    childToJointTf.set_translation(np.random.rand(3, 1))

    joint.setTransformFromParentBodyNode(parentToJointTf)
    joint.setTransformFromChildBodyNode(childToJointTf)

    storedParentTf = joint.getTransformFromParentBodyNode()
    storedChildTf = joint.getTransformFromChildBodyNode()

    assert np.allclose(parentToJointTf.matrix(), storedParentTf.matrix())
    assert np.allclose(childToJointTf.matrix(), storedChildTf.matrix())


def test_BallJoint_positions_conversion():
    assert np.allclose(
        dart.dynamics.BallJoint.convertToPositions(np.eye(3)),
        np.zeros((1, 3))
    )
    assert np.allclose(
        dart.dynamics.BallJoint.convertToPositions(
            np.array([[0, 1, 0], [-1, 0, 0], [0, 0, 1]])),
        np.array([0, 0, -np.pi/2])
    )

    for i in range(30):
        ballJointPos = np.random.uniform(-np.pi/2, np.pi/2, 3)
        assert np.allclose(
            dart.dynamics.BallJoint.convertToRotation(
                dart.dynamics.BallJoint.convertToPositions(
                    dart.dynamics.BallJoint.convertToRotation(ballJointPos)
                )),
            dart.dynamics.BallJoint.convertToRotation(ballJointPos)
        )


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
import dartpy as dart
import numpy as np


def collision_groups_tester(cd):
    size = [1, 1, 1]
    pos1 = [0, 0, 0]
    pos2 = [0.5, 0, 0]

    simple_frame1 = dart.dynamics.SimpleFrame()
    simple_frame2 = dart.dynamics.SimpleFrame()

    sphere1 = dart.dynamics.SphereShape(1)
    sphere2 = dart.dynamics.SphereShape(1)

    simple_frame1.setShape(sphere1)
    simple_frame2.setShape(sphere2)

    group = cd.createCollisionGroup()
    group.addShapeFrame(simple_frame1)
    group.addShapeFrame(simple_frame2)
    assert group.getNumShapeFrames() is 2

    #
    #    ( s1,s2 )              collision!
    # ---+---|---+---+---+---+--->
    #   -1   0  +1  +2  +3  +4
    #
    assert group.collide()

    #
    #    (  s1   )   (  s2   )  no collision
    # ---+---|---+---+---+---+--->
    #   -1   0  +1  +2  +3  +4
    #
    simple_frame2.setTranslation([3, 0, 0])
    assert not group.collide()

    option = dart.collision.CollisionOption()
    result = dart.collision.CollisionResult()

    group.collide(option, result)
    assert not result.isCollision()
    assert result.getNumContacts() is 0

    option.enableContact = True
    simple_frame2.setTranslation([1.99, 0, 0])

    group.collide(option, result)
    assert result.isCollision()
    assert result.getNumContacts() is not 0

    # Repeat the same test with BodyNodes instead of SimpleFrames

    group.removeAllShapeFrames()
    assert group.getNumShapeFrames() is 0

    skel1 = dart.dynamics.Skeleton()
    skel2 = dart.dynamics.Skeleton()

    [joint1, body1] = skel1.createFreeJointAndBodyNodePair(None)
    [joint2, body2] = skel2.createFreeJointAndBodyNodePair(None)

    shape_node1 = body1.createShapeNode(sphere1)
    shape_node1.createVisualAspect()
    shape_node1.createCollisionAspect()

    shape_node2 = body2.createShapeNode(sphere2)
    shape_node2.createVisualAspect()
    shape_node2.createCollisionAspect()

    group.addShapeFramesOf(body1)
    group.addShapeFramesOf(body2)

    assert group.getNumShapeFrames() is 2

    assert group.collide()

    joint2.setPosition(3, 3)
    assert not group.collide()

    # Repeat the same test with BodyNodes and two groups

    joint2.setPosition(3, 0)

    group.removeAllShapeFrames()
    assert group.getNumShapeFrames() is 0
    group2 = cd.createCollisionGroup()

    group.addShapeFramesOf(body1)
    group2.addShapeFramesOf(body2)

    assert group.getNumShapeFrames() is 1
    assert group2.getNumShapeFrames() is 1

    assert group.collide(group2)

    joint2.setPosition(3, 3)
    assert not group.collide(group2)


def test_collision_groups():
    cd = dart.collision.FCLCollisionDetector()
    collision_groups_tester(cd)

    cd = dart.collision.DARTCollisionDetector()
    collision_groups_tester(cd)

    if hasattr(dart.collision, "BulletCollisionDetector"):
        cd = dart.collision.BulletCollisionDetector()
        collision_groups_tester(cd)

    if hasattr(dart.collision, "OdeCollisionDetector"):
        cd = dart.collision.OdeCollisionDetector()
        collision_groups_tester(cd)


# TODO: Add more collision detectors
@pytest.mark.parametrize("cd", [dart.collision.FCLCollisionDetector()])
def test_filter(cd):
    # Create two bodies skeleton. The two bodies are placed at the same position
    # with the same size shape so that they collide by default.
    skel = dart.dynamics.Skeleton()

    shape = dart.dynamics.BoxShape(np.ones(3))

    _, body0 = skel.createRevoluteJointAndBodyNodePair()
    shape_node0 = body0.createShapeNode(shape)
    shape_node0.createVisualAspect()
    shape_node0.createCollisionAspect()

    _, body1 = skel.createRevoluteJointAndBodyNodePair(body0)
    shape_node1 = body1.createShapeNode(shape)
    shape_node1.createVisualAspect()
    shape_node1.createCollisionAspect()

    # Create a world and add the created skeleton
    world = dart.simulation.World()
    world.addSkeleton(skel)

    # Set a new collision detector
    constraint_solver = world.getConstraintSolver()
    constraint_solver.setCollisionDetector(cd)

    # Get the collision group from the constraint solver
    group = constraint_solver.getCollisionGroup()
    assert group.getNumShapeFrames() == 2

    # Create BodyNodeCollisionFilter
    option = constraint_solver.getCollisionOption()
    body_node_filter = dart.collision.BodyNodeCollisionFilter()
    option.collisionFilter = body_node_filter

    skel.enableSelfCollisionCheck()
    skel.enableAdjacentBodyCheck()
    assert skel.isEnabledSelfCollisionCheck()
    assert skel.isEnabledAdjacentBodyCheck()
    assert group.collide()
    assert group.collide(option)

    skel.enableSelfCollisionCheck()
    skel.disableAdjacentBodyCheck()
    assert skel.isEnabledSelfCollisionCheck()
    assert not skel.isEnabledAdjacentBodyCheck()
    assert group.collide()
    assert not group.collide(option)

    skel.disableSelfCollisionCheck()
    skel.enableAdjacentBodyCheck()
    assert not skel.isEnabledSelfCollisionCheck()
    assert skel.isEnabledAdjacentBodyCheck()
    assert group.collide()
    assert not group.collide(option)

    skel.disableSelfCollisionCheck()
    skel.disableAdjacentBodyCheck()
    assert not skel.isEnabledSelfCollisionCheck()
    assert not skel.isEnabledAdjacentBodyCheck()
    assert group.collide()
    assert not group.collide(option)

    # Test collision body filtering
    skel.enableSelfCollisionCheck()
    skel.enableAdjacentBodyCheck()
    body_node_filter.addBodyNodePairToBlackList(body0, body1)
    assert not group.collide(option)
    body_node_filter.removeBodyNodePairFromBlackList(body0, body1)
    assert group.collide(option)
    body_node_filter.addBodyNodePairToBlackList(body0, body1)
    assert not group.collide(option)
    body_node_filter.removeAllBodyNodePairsFromBlackList()
    assert group.collide(option)


def test_raycast():
    cd = dart.collision.BulletCollisionDetector()

    simple_frame = dart.dynamics.SimpleFrame()
    sphere = dart.dynamics.SphereShape(1)
    simple_frame.setShape(sphere)

    group = cd.createCollisionGroup()
    group.addShapeFrame(simple_frame)
    assert group.getNumShapeFrames() == 1

    option = dart.collision.RaycastOption()
    option.mEnableAllHits = False

    result = dart.collision.RaycastResult()
    assert not result.hasHit()

    ray_hit = dart.collision.RayHit()

    result.clear()
    simple_frame.setTranslation(np.zeros(3))
    assert group.raycast([-2, 0, 0], [2, 0, 0], option, result)
    assert result.hasHit()
    assert len(result.mRayHits) == 1
    ray_hit = result.mRayHits[0]
    assert np.isclose(ray_hit.mPoint, [-1, 0, 0]).all()
    assert np.isclose(ray_hit.mNormal, [-1, 0, 0]).all()
    assert ray_hit.mFraction == pytest.approx(0.25)

    result.clear()
    simple_frame.setTranslation(np.zeros(3))
    assert group.raycast([2, 0, 0], [-2, 0, 0], option, result)
    assert result.hasHit()
    assert len(result.mRayHits) == 1
    ray_hit = result.mRayHits[0]
    assert np.isclose(ray_hit.mPoint, [1, 0, 0]).all()
    assert np.isclose(ray_hit.mNormal, [1, 0, 0]).all()
    assert ray_hit.mFraction == pytest.approx(0.25)

    result.clear()
    simple_frame.setTranslation([1, 0, 0])
    assert group.raycast([-2, 0, 0], [2, 0, 0], option, result)
    assert result.hasHit()
    assert len(result.mRayHits) == 1
    ray_hit = result.mRayHits[0]
    assert np.isclose(ray_hit.mPoint, [0, 0, 0]).all()
    assert np.isclose(ray_hit.mNormal, [-1, 0, 0]).all()
    assert ray_hit.mFraction == pytest.approx(0.5)


if __name__ == "__main__":
    pytest.main()



import pytest
import dartpy as dart


def test_basics():
    dart.common.trace("trace log")
    dart.common.debug("debug log")
    dart.common.info("info log")
    dart.common.warn("warn log")
    dart.common.error("error log")
    dart.common.fatal("fatal log")


def test_arguments():
    val = 10
    dart.common.info("Log with param '{}' and '{}'".format(1, val))


if __name__ == "__main__":
    pytest.main()



import pytest
import math
import dartpy as dart


# Problem reference: http://ab-initio.mit.edu/wiki/index.php/NLopt_Tutorial
class SampleObjFunc(dart.optimizer.Function):
    def eval(self, x):
        return math.sqrt(x[1])

    def evalGradient(self, x, grad):
        grad[0] = 0
        grad[1] = 0.5 / (math.sqrt(x[1]) + 0.000001)


class SampleConstFunc(dart.optimizer.Function):
    def __init__(self, a, b):
        super(SampleConstFunc, self).__init__()
        self.a = a
        self.b = b

    def eval(self, x):
        return (self.a*x[0] + self.b) * (self.a*x[0] + self.b) * (self.a*x[0] + self.b) - x[1];

    def evalGradient(self, x, grad):
        grad[0] = 3 * self.a * (self.a*x[0] + self.b) * (self.a*x[0] + self.b);
        grad[1] = -1.0;


def test_gradient_descent_solver():
    prob = dart.optimizer.Problem(2)
    assert prob.getDimension() is 2

    prob.setLowerBounds([-1e100, 0])
    prob.setInitialGuess([1.234, 5.678])

    assert prob.getInitialGuess()[0] == pytest.approx(1.234)

    obj = SampleObjFunc()
    prob.setObjective(obj)

    solver = dart.optimizer.GradientDescentSolver(prob)
    success = solver.solve()
    assert success is True

    min_f = prob.getOptimumValue()
    opt_x = prob.getOptimalSolution()

    assert min_f == pytest.approx(0, 1e-6)
    assert len(opt_x) == prob.getDimension()
    assert opt_x[0] == pytest.approx(1.234, 0.0)
    assert opt_x[1] == pytest.approx(0, solver.getTolerance())


def test_nlopt_solver():
    if not hasattr(dart.optimizer, 'NloptSolver'):
        return

    prob = dart.optimizer.Problem(2)
    assert prob.getDimension() is 2

    prob.setLowerBounds([-1e100, 0])
    prob.setInitialGuess([1.234, 5.678])

    assert prob.getInitialGuess()[0] == pytest.approx(1.234)

    obj = SampleObjFunc()
    prob.setObjective(obj)

    const1 = SampleConstFunc(2, 0)
    const2 = SampleConstFunc(-1, 1)
    prob.addIneqConstraint(const1)
    prob.addIneqConstraint(const2)

    solver = dart.optimizer.NloptSolver(prob)
    solver.setAlgorithm(dart.optimizer.NloptSolver.Algorithm.LD_MMA)
    success = solver.solve()
    assert success is True

    min_f = prob.getOptimumValue()
    opt_x = prob.getOptimalSolution()

    assert min_f == pytest.approx(0.544330847, 1e-5)
    assert len(opt_x) == prob.getDimension()
    assert opt_x[0] == pytest.approx(0.333334, 1e-5)
    assert opt_x[1] == pytest.approx(0.296296, 1e-5)


if __name__ == "__main__":
    pytest.main()



import numpy as np
import dartpy as dart


class MyWorldNode(dart.gui.osg.RealTimeWorldNode):
    def __init__(self, world):
        super(MyWorldNode, self).__init__(world)

    def customPreStep(self):
        pass


def main():
    world = dart.utils.SkelParser.readWorld("dart://sample/skel/chain.skel")
    world.setGravity([0, -9.81, 0])

    chain = world.getSkeleton(0)
    dof = chain.getNumDofs()

    # Set initial pose
    init_pose = np.zeros(dof)
    for i in range(dof):
        init_pose[i] = dart.math.Random.uniform(-0.5, 0.5)
    chain.setPositions(init_pose)

    # Set joint dampings
    for i in range(chain.getNumJoints()):
        joint = chain.getJoint(i)
        for j in range(joint.getNumDofs()):
            joint.setDampingCoefficient(j, 0.01)

    node = MyWorldNode(world)

    viewer = dart.gui.osg.Viewer()
    viewer.addWorldNode(node)

    # Grid settings
    grid = dart.gui.osg.GridVisual()
    grid.setPlaneType(dart.gui.osg.GridVisual.PlaneType.ZX)
    grid.setOffset([0, -0.55, 0])
    viewer.addAttachment(grid)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([0.8, 0.0, 0.8], [0, -0.25, 0], [0, 0.5, 0])
    viewer.run()


if __name__== "__main__":
    main()



# Copyright (c) 2011-2022, The DART development contributors
# All rights reserved.
#
# The list of contributors can be found at:
#   https://github.com/dartsim/dart/blob/master/LICENSE
#
# This file is provided under the "BSD-style" License

import pytest
import dartpy as dart


def test_case_conversions():
    assert dart.common.toUpper("to UppEr") == "TO UPPER"
    assert dart.common.toLower("to LowEr") == "to lower"


def test_trim():
    assert dart.common.trimLeft(" trim ThIs ") == "trim ThIs "
    assert dart.common.trimRight(" trim ThIs ") == " trim ThIs"
    assert dart.common.trim(" trim ThIs ") == "trim ThIs"

    assert dart.common.trimLeft("\n trim ThIs ", " ") == "\n trim ThIs "
    assert dart.common.trimLeft("\n trim ThIs ", "\n") == " trim ThIs "
    assert dart.common.trimRight(" trim ThIs \n", " ") == " trim ThIs \n"
    assert dart.common.trimRight(" trim ThIs \n", "\n") == " trim ThIs "
    assert dart.common.trim("\n trim ThIs \n", " ") == "\n trim ThIs \n"
    assert dart.common.trim("\n trim ThIs \n", "\n") == " trim ThIs "

    assert dart.common.trimLeft("\n trim ThIs \n", " \n") == "trim ThIs \n"
    assert dart.common.trimRight("\n trim ThIs \n", " \n") == "\n trim ThIs"
    assert dart.common.trim("\n trim ThIs \n", " \n") == "trim ThIs"


def test_split():
    assert len(dart.common.split(" trim ThIs ")) == 2
    assert dart.common.split(" trim ThIs ")[0] == "trim"
    assert dart.common.split(" trim ThIs ")[1] == "ThIs"


if __name__ == "__main__":
    pytest.main()



import dartpy as dart


def main():
    world = dart.simulation.World()

    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    ground = urdfParser.parseSkeleton("dart://sample/urdf/KR5/ground.urdf")
    world.addSkeleton(kr5)
    world.addSkeleton(ground)
    print('Robot {} is loaded'.format(kr5.getName()))

    for i in range(100):
        if i % 10 == 0:
            print('[{}] joint position: {}'.format(world.getSimFrames(), kr5.getPositions()))
        world.step()


if __name__ == "__main__":
    main()



import platform
import pytest
import dartpy as dart
import numpy as np


def test_ball_joint_constraint():
    world = dart.utils.SkelParser.readWorld('dart://sample/skel/chain.skel')
    world.setGravity([0, -9.81, 0])
    world.setTimeStep(1.0/2000)

    # Set joint damping
    chain = world.getSkeleton(0)
    for i in range(chain.getNumJoints()):
        joint = chain.getJoint(i)
        for j in range(joint.getNumDofs()):
            joint.setDampingCoefficient(j, 0.01)

    # Create a ball joint contraint
    bd1 = chain.getBodyNode('link 6')
    bd2 = chain.getBodyNode('link 10')
    offset1 = [0, 0.025, 0]
    joint_pos = bd1.getTransform().multiply(offset1)
    offset2 = bd2.getTransform().inverse().multiply(joint_pos)
    constraint = dart.constraint.BallJointConstraint(bd1, bd2, joint_pos)
    assert constraint.getType() == dart.constraint.BallJointConstraint.getStaticType()

    # Add ball joint constraint to the constraint solver
    constraint_solver = world.getConstraintSolver()
    constraint_solver.addConstraint(constraint)

    # Check if the ball joint constraint is being satisfied
    for _ in range(100):
        world.step()
        pos1 = bd1.getTransform().multiply(offset1)
        pos2 = bd2.getTransform().multiply(offset2)
        assert np.isclose(pos1, pos2).all()


if __name__ == "__main__":
    pytest.main()



import platform

import pytest
import dartpy as dart
import os

from tests.util import get_asset_path


def test_parse_fetch():
    assert dart.utils.MjcfParser.readWorld(
        'dart://sample/mjcf/openai/robotics/fetch/pick_and_place.xml') is not None


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
import numpy as np
import dartpy as dart


def test_basic():
    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    assert kr5 is not None

    for i in range(kr5.getNumBodyNodes()):
        body = kr5.getBodyNode(i)
        body_force = body.getBodyForce()
        assert body_force.size == 6
        bodyPtr = body.getBodyNodePtr()
        assert body == bodyPtr
        assert body.getName() == bodyPtr.getName()
        assert np.array_equal(
            np.array(body.getSpatialVelocity()), np.zeros(6)) is True
        shape_nodes = body.getShapeNodes()
        for shape_node in shape_nodes:
            print(shape_node)
            if shape_node.hasVisualAspect():
                visual = shape_node.getVisualAspect()
                visual.getRGBA()
            if shape_node.hasCollisionAspect():
                collision = shape_node.getCollisionAspect()
            if shape_node.hasDynamicsAspect():
                dynamics = shape_node.getDynamicsAspect()


def test_get_child_methods():
    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    assert kr5 is not None

    currentBodyNode = kr5.getRootBodyNode()
    assert currentBodyNode is not None

    for i in range(1, kr5.getNumBodyNodes()):
        childBodyNode = currentBodyNode.getChildBodyNode(0)
        childJoint = currentBodyNode.getChildJoint(0)

        assert childBodyNode is not None
        assert childJoint is not None
        assert childBodyNode.getName() == kr5.getBodyNode(i).getName()
        assert childJoint.getName() == kr5.getJoint(i).getName()

        currentBodyNode = childBodyNode


def test_get_inertia():
    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    assert kr5 is not None

    inertias = [kr5.getBodyNode(i).getInertia()
                for i in range(1, kr5.getNumBodyNodes())]
    assert all([inertia is not None for inertia in inertias])


if __name__ == "__main__":
    pytest.main()



import numpy as np
import dartpy as dart


class HelloWorldNode(dart.gui.osg.RealTimeWorldNode):
    def __init__(self, world, kr5):
        super(HelloWorldNode, self).__init__(world)
        self.kr5 = kr5
        self.dofs = self.kr5.getNumDofs()
        self.ee = kr5.getBodyNode('palm')
        self.Kp = np.eye(3) * 50.0
        self.Kd = np.eye(self.dofs) * 5.0
        self.ee_offset = [0.05, 0, 0]

        tf = self.ee.getTransform()
        tf.pretranslate(self.ee_offset)
        self.target = dart.dynamics.SimpleFrame(dart.dynamics.Frame.World(), "target", tf)

    def customPreStep(self):
        M = self.kr5.getMassMatrix()

        J = self.ee.getLinearJacobian()
        Jt = J.transpose();
        JJt = np.matmul(J, Jt)
        kI = 0.0025 * np.eye(3)
        invJ = np.matmul(Jt, np.linalg.inv(JJt + kI))

        dJ = self.ee.getLinearJacobianDeriv()
        dJt = dJ.transpose()
        dJdJt = np.matmul(dJ, dJt)
        invdJ = np.matmul(dJt, np.linalg.inv(dJdJt + kI))

        e = self.target.getTransform().translation() - self.ee.getTransform().translation()
        de = -self.ee.getLinearVelocity()

        cg = self.kr5.getCoriolisAndGravityForces()

        tmp1 = np.matmul(np.matmul(invJ, self.Kp), de)
        tmp2 = np.matmul(np.matmul(invdJ, self.Kp), e)

        forces1 = np.matmul(M, tmp1 + tmp2)
        forces2 = cg
        forces3 = np.matmul(np.matmul(np.matmul(self.Kd, invJ), self.Kp), e)

        forces = forces1 + forces2 + forces3

        self.kr5.setForces(forces)


def main():
    world = dart.simulation.World()

    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    ground = urdfParser.parseSkeleton("dart://sample/urdf/KR5/ground.urdf")
    world.addSkeleton(kr5)
    world.addSkeleton(ground)
    world.setGravity([0, -9.81, 0])

    node = HelloWorldNode(world, kr5)

    # Create world node and add it to viewer
    viewer = dart.gui.osg.Viewer()
    viewer.addWorldNode(node)

    # Grid settings
    grid = dart.gui.osg.GridVisual()
    grid.setPlaneType(dart.gui.osg.GridVisual.PlaneType.ZX)
    grid.setOffset([0, -0.55, 0])
    viewer.addAttachment(grid)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([2.0, 1.0, 2.0],
                                 [0.00, 0.00, 0.00],
                                 [-0.24, 0.94, -0.25])
    viewer.run()


if __name__ == "__main__":
    main()
